#-*- coding: utf-8 -*-

#**********************************************
# oct2spec/Description.py
#
# Class file for the class Description
#
# Based on R2spec by Pierre-Yves chibon
#
# Copyright (C) 2011 Orion Poplawski <orion@cora.nwra.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#**********************************************

import os, re, shutil, string

class Description:
    '''
    Description class.
        Creates and handle the object Description
    '''

    def __init__(self):
        ''' Creates the object '''
        self.license = None
        self.version = None
        self.description = None
        self.summary = None
        self.file = None

    def setVersion(self):
        ''' Set the version of the package '''
        # Retrieve the Version number
        try:
            self.version = self.file['version'].replace('-','.')
        except KeyError as err:
            print("No version set")
            self.version = ""


    def setLicense(self):
        ''' Returns the license of the package '''
        try:
            self.license = re.sub(' or later','+',re.sub(' version ','v',self.file['license']))
        except IndexError as err:
            print("No license set")
            self.license = ""


    def setDescription(self):
        ''' 
        Returns the description of the package. 
        Catch the description in one or several lines 
        '''
        try:
            self.description = self.formatDescription(self.file['description'])
            print(self.description)
        except KeyError as err:
            print('No Description available')
            self.description = ""


    def setSummary(self):
        ''' 
        Returns the summary of the package. 
        Catch the title in one or several lines 
        '''
        try:
            self.summary = self.file['title'][:56].capitalize()
            if self.summary.endswith('.'):
                self.summary = self.summary[:-1]
        except KeyError as err:
            print('No Title available')
            self.summary = ""

    def processDependencieList(self, deplist):
        '''
        Add the prefix octave- to the dependencies
        Check if the dependencies does not contains > or = or a number
        '''
        # Package that should not be taken into account for the dependencies
        ignorelist = ['octave']
        # Regular expression used to determine whether the string is a version number
        versionmotif = re.compile('\d\.\d\.?\d?')
               
        cnt = 0
        while cnt < len(deplist):
            # remove empty dep
            deplist[cnt] = deplist[cnt].strip()
            if deplist[cnt] == "":
                del deplist[cnt]
                continue
            print('Processing dep <%s>'%deplist[cnt])
            if deplist[cnt].strip() in ignorelist:
                del deplist[cnt]
                if ">" in deplist[cnt]:
                    del deplist[cnt]
                if len( versionmotif.findall(deplist[cnt]) ):
                    del deplist[cnt]
                continue

            # add the octave- prefix when needed
            if  not ">" in deplist[cnt] \
                and not "=" in deplist[cnt] \
                and len( versionmotif.findall(deplist[cnt]) ) == 0 \
                and deplist[cnt].strip() not in ignorelist:
                    deplist[cnt] = "octave-%s" %deplist[cnt].strip()
            cnt = cnt + 1
        return " ".join(deplist)

    def setRequires(self):
        ''' 
        Returns the required libraries of the package.
        Catch the requirement in one or more lines.
        '''
        # Characters to change and to change to 
        char = { 
                '\r':'', 
                '(' :' ', 
                ')' :' ', 
                ',' :' ',
                '  ':' ',
                }

        
        # Retrieve the Depends
        try:
            depends = self.file['depends']
            for c in list(char.keys()):
                depends = depends.replace(c, char[c])
            
            # Check that the dependencies are not ''
            deplist = depends.split(' ')
            self.depends = self.processDependencieList(deplist).strip()
        except KeyError as err:
            print('No Depends set')
            self.depends = ""

        # Retrieve the Suggests
        try:
            suggests = self.file['suggests']
            for c in list(char.keys()):
                suggests = suggests.replace(c, char[c])
            
            # Check that the dependencies are not ''
            suglist = suggests.split(' ')
            self.suggests = self.processDependencieList(suglist).strip()
        except KeyError as err:
            print('No Suggests set')
            self.suggests = ""

  
    def setDescriptionFile(self, package):
        ''' Returns the description file of the package '''

        # Save the top directory
        original_directory = os.getcwd()
        #print os.getcwd()
        # Change directory and load the DESCRIPTION file in memory
        os.chdir(package.tardir)
        self.file = self.parseConfig('DESCRIPTION')

        # Comes back to the original directory
        os.chdir(original_directory)


    def parseConfig(self, filename):
        ''' Returns a dictionnary of all the options available '''
        # Character used in the DESCRIPTION file
        COMMENT_CHAR = '#' # char for comments
        OPTION_CHAR =  ':' # char used to separate option from value
        
        options = {}
        f = open(filename)
        for line in f:
            # First, remove comments:
            if COMMENT_CHAR in line:
                # split on comment char, keep only the part before
                line, comment = str(line, errors='ignore').split(COMMENT_CHAR, 1)
            # Second, find lines with an option=value:
            if OPTION_CHAR in line:
                # split on option char:
                option, value = str(line, errors='ignore').split(OPTION_CHAR, 1)
                # strip spaces:
                option = option.strip().lower()
                value = value.strip()
                # store in dictionary:
                options[option] = value.encode('UTF-8')
            else:
                #values = unicode(line, errors='ignore').split(" ")
                values = line.split(" ")
                for value in values:
                    if value.strip() != "":
                        options[option] += " " +value.strip().encode('UTF-8')
                
        f.close()
        return options
    
    def formatDescription(self, description):
        ''' Format the description as required by rpm '''
        step = 75
        cnt = 0
        out = []
        char = 0
        while cnt < len(description) and char < description.rfind(" "):
            if len(description[cnt:]) >= step:
                char = description[cnt : cnt + step ].rfind(" ")  + cnt
                out.append(description[cnt:char])
                cnt = char + 1
            else:
                out.append(description[cnt:])
                cnt += len(description[cnt:])

        return "\n".join(out)
