#-*- coding: utf-8 -*-

#***********************************************
# oct2spec/oct2rpm.py
#
# oct2rpm generates RPM using the oct2spec library
# building the RPM and adjusting the spec accordingly
#
# Based on R2spec by Pierre-Yves chibon
#
# Copyright (C) 2011 Orion Poplawski <orion@cora.nwra.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#***********************************************

import getopt, sys, re, os
from . import OctPackage
from . import Build
from . import Spec

USAGE ="""
This program is designed to create RPM from a source file or a url using oct2spec.
Usage:  oct2rpm -u <url> 
        oct2rpm -s <source>
        oct2rpm -s folder/folder/<source>
        oct2rpm -p <name>
        oct2rpm -p <name> --mock=fedora-12-i386

The source can be in the same directory or in a subdirectory.

The information about the packager:
- Fill /etc/oct2spec.conf 
- Copy and fill /etc/oct2spec to ~/.oct2spec.conf
- Use the option -n/--name and -e/--email to set the corresponding values
"""

#################################################
### main
# Do the job :)
class oct2rpm:
    """ oct2rpm class
            Enables to use this code from another script
    """
    
    def __init__(self):
        ''' Instanciate the default values for the attributes '''
        self.built = []
    
    def main(self, source, url, package, forge, \
            copyFile, dependencies, name, email, force, mock):
        ''' Performs the work'''
        
        from . import Oct2spec
        
        # Generate the spec file
        r = OctPackage.OctPackage()
        # Determine where to generate the spec according to the rpmmacro
        specdir = r.getRPMTopDirectory("_specdir", package)
        
        # Go to the folder
        if specdir is not None:
            os.chdir(specdir)
        else:
            print("Cannot change to the spec folder, using the current folder instead.")
        # Enforce the copyfile and the force option
        r = oct2spec()
        r.todo = False
        r.usage = USAGE
        specname = r.main(source, url, package, forge, True, name, email, True)
        specname = specname + ".spec"
        self.built.append(specname)
        
        print("*"*50)
        print("** Spec generated, starting RPM construction")
        # Remove the content of the %file section
        spec = Spec()
        spec.readSpec(specname)
        spec.cleanFiles()
        spec.writeSpecTosystem(specname)
        # Generate the rpm from the cleaned spec file
        rpmarg = r.package.packager.rpmarg
        self.generateRpm(spec, specname, dependencies, rpmarg, mock)
        
        
    def generateRpm(self, spec, specname, dependencies = False, rpmarg ='ba', mock = False):
        ''' Generate the rpm '''
        from oct2spec.oct2rpmError import oct2rpmError, BuildError, BuildDepencenciesError
        
        # Build
        b = Build()
        try:
            # First build
            b.build(specname, rpmarg, mock)
            # Parse the output
            files = b.parseError(specname)
            if len(files) != 0:
                # Add the file to the file section
                spec.addFile(files)
                # Write down the spec file
                spec.writeSpecTosystem(specname)
            # Rebuild the package when build has failed the first time
            if b.outcode: # == 1 when build failed and 0 when build passed
                self.generateRpm(spec, specname, dependencies, rpmarg, mock)
        except BuildDepencenciesError as err:
            print("An error occured during the build")
            print(err)
            if dependencies:
                print("\n* Building dependencies")
                for package in b.deps:
                    print(package)
            sys.exit(1)
        except BuildError as err:
            print("An error occured during the build")
            print(err)
            sys.exit(1)
        
        if b.outcode == 0 :
            print("** RPM %s done **" %specname.split('.spec')[0])
            # Get the list of rpm generated
            b.getRpm(specname)
            print("\n".join(b.rpm))
